@@ -3,20 +3,21 @@
 from uno import getComponentContext
 getComponentContext()
 import argparse
-import gettext
 import pkg_resources
 from concurrent.futures import ProcessPoolExecutor, as_completed
 from datetime import datetime, date, timedelta
+from gettext import translation, install
 from multiprocessing import cpu_count
 
-from unogenerator.commons import __version__, addDebugSystem, argparse_epilog, Colors, Coord as C
+from unogenerator.commons import __version__, addDebugSystem, argparse_epilog, ColorsNamed, Coord as C
 from unogenerator.reusing.currency import Currency
 from unogenerator.reusing.percentage import Percentage
 from unogenerator.unogenerator import ODT_Standard, ODS_Standard
+from unogenerator.helpers import helper_values_with_total, helper_totals_row, helper_totals_column
 from os import remove
 
 try:
-    t=gettext.translation('unogenerator',pkg_resources.resource_filename("unogenerator","locale"))
+    t=translation('unogenerator', pkg_resources.resource_filename("unogenerator","locale"))
     _=t.gettext
 except:
     _=str
@@ -65,9 +66,9 @@ def main(arguments=None):
        
 def demo_ods_standard(language):
     if language=="en":
-        lang1=gettext.install('unogenerator', 'badlocale')
+        lang1=install('unogenerator', 'badlocale')
     else:
-        lang1=gettext.translation('unogenerator', 'unogenerator/locale', languages=[language])
+        lang1=translation('unogenerator', 'unogenerator/locale', languages=[language])
         lang1.install()
     
     doc=ODS_Standard()
@@ -79,51 +80,71 @@ def demo_ods_standard(language):
         ["unogenerator", "demo", "files"]
     )
     doc.createSheet("Styles", 1)
-    doc.setColumnsWidth([2000, 5000, 2000,  2000,  2000,  2000,  5000,  5000,  5000,  5000,  5000,  5000])
+    doc.setColumnsWidth([5000, 5000, 2000,  2000,  2000,  2000,  5000,  5000,  5000,  5000,  5000,  5000])
     
-    doc.addCellWithStyle("A1", _("Style name"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("B1", _("Date and time"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("C1", _("Date"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("D1", _("Integer"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("E1", _("Euros"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("F1", _("Dollars"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("G1", _("Percentage"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("H1", _("Number with 2 decimals"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("I1", _("Number with 6 decimals"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("J1", _("Time"), Colors["Orange"], "BoldCenter")
-    doc.addCellWithStyle("K1", _("Boolean"), Colors["Orange"], "BoldCenter")
-    for row, color_key in enumerate(Colors.keys()):
-        doc.addCellWithStyle(C("A2").addRow(row), color_key, Colors[color_key], "Bold")
-        doc.addCellWithStyle(C("B2").addRow(row), datetime.now(), Colors[color_key], "Datetime")
-        doc.addCellWithStyle(C("C2").addRow(row), date.today(), Colors[color_key], "Date")
-        doc.addCellWithStyle(C("D2").addRow(row), pow(-1, row)*-10000000, Colors[color_key], "Integer")
-        doc.addCellWithStyle(C("E2").addRow(row), Currency(pow(-1, row)*12.56, "EUR"), Colors[color_key], "EUR")
-        doc.addCellWithStyle(C("F2").addRow(row), Currency(pow(-1, row)*12345.56, "USD"), Colors[color_key], "USD")
-        doc.addCellWithStyle(C("G2").addRow(row), Percentage(pow(-1, row)*1, 3), Colors[color_key],  "Percentage")
-        doc.addCellWithStyle(C("H2").addRow(row), pow(-1, row)*123456789.121212, Colors[color_key], "Float6")
-        doc.addCellWithStyle(C("I2").addRow(row), pow(-1, row)*-12.121212, Colors[color_key], "Float2")
-        doc.addCellWithStyle(C("J2").addRow(row), (datetime.now()+timedelta(seconds=3600*12*row)).time(), Colors[color_key], "Time")
-        doc.addCellWithStyle(C("K2").addRow(row), bool(row%2), Colors[color_key], "Bool")
-        
-    doc.addCellWithStyle("E10","=sum(E2:E8)", Colors["GrayLight"], "EUR" )
-    doc.addCellMerged("E12:K12", "Prueba de merge", Colors["Yellow"], style="BoldCenter")
-    doc.setComment("B11", "This is nice comment")
+    doc.addCellWithStyle("A1", _("Style name"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("B1", _("Date and time"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("C1", _("Date"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("D1", _("Integer"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("E1", _("Euros"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("F1", _("Dollars"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("G1", _("Percentage"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("H1", _("Number with 2 decimals"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("I1", _("Number with 6 decimals"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("J1", _("Time"), ColorsNamed.Orange, "BoldCenter")
+    doc.addCellWithStyle("K1", _("Boolean"), ColorsNamed.Orange, "BoldCenter")
+    colors_list=([a for a in dir(ColorsNamed()) if not a.startswith('__')])
+    for row, color_str in enumerate(colors_list):
+        color_key=getattr(ColorsNamed(), color_str)
+        doc.addCellWithStyle(C("A2").addRow(row), color_str, color_key, "Bold")
+        doc.addCellWithStyle(C("B2").addRow(row), datetime.now(), color_key, "Datetime")
+        doc.addCellWithStyle(C("C2").addRow(row), date.today(), color_key, "Date")
+        doc.addCellWithStyle(C("D2").addRow(row), pow(-1, row)*-10000000, color_key, "Integer")
+        doc.addCellWithStyle(C("E2").addRow(row), Currency(pow(-1, row)*12.56, "EUR"), color_key, "EUR")
+        doc.addCellWithStyle(C("F2").addRow(row), Currency(pow(-1, row)*12345.56, "USD"), color_key, "USD")
+        doc.addCellWithStyle(C("G2").addRow(row), Percentage(pow(-1, row)*1, 3), color_key,  "Percentage")
+        doc.addCellWithStyle(C("H2").addRow(row), pow(-1, row)*123456789.121212, color_key, "Float6")
+        doc.addCellWithStyle(C("I2").addRow(row), pow(-1, row)*-12.121212, color_key, "Float2")
+        doc.addCellWithStyle(C("J2").addRow(row), (datetime.now()+timedelta(seconds=3600*12*row)).time(), color_key, "Time")
+        doc.addCellWithStyle(C("K2").addRow(row), bool(row%2), color_key, "Bool")
+
+    doc.addCellWithStyle(C("E2").addRow(row+1),f"=sum(E2:{C('E2').addRow(row).string()})", ColorsNamed.GrayLight, "EUR" )
+    doc.addCellMergedWithStyle("E15:K15", "Prueba de merge", ColorsNamed.Yellow, style="BoldCenter")
+    doc.setComment("B14", "This is nice comment")
     
     doc.freezeAndSelect("B2")
+
+    ## HELPERS
+    doc.createSheet("Helpers", 2)
+    doc.addCellMergedWithStyle("A1:E1","Helper values with total (horizontal)", ColorsNamed.Orange, "BoldCenter")
+    helper_values_with_total(doc,"A2", "Suma 3", [1,2,3],  horizontal=True)
+    doc.addCellMergedWithStyle("A4:E4","Helper values with total (vertical)", ColorsNamed.Orange, "BoldCenter")
+    helper_values_with_total(doc,"A5", "Suma 3", [1,2,3],  horizontal=False)
+    doc.addCellMergedWithStyle("A11:C11","List of rows", ColorsNamed.Orange, "BoldCenter")
+    doc.addListOfRowsWithStyle("A12", [[1,2,3],[4,5,6],[7,8,9]], ColorsNamed.White)
+    doc.addCellMergedWithStyle("E11:G11","List of columns", ColorsNamed.Orange, "BoldCenter")
+    doc.addListOfColumnsWithStyle("E12", [[1,2,3],[4,5,6],[7,8,9]], ColorsNamed.White)
+
+    helper_totals_row(doc, "A17", ["#SUM"]*3, list_of_styles=None, row_from="12", row_to="15")
+    helper_totals_column(doc, "I12", ["#SUM"]*3, list_of_styles=None, column_from="E", column_to="G")
+
     doc.removeSheet(0)
     doc.save(f"unogenerator_example_{language}.ods")
     doc.export_xlsx(f"unogenerator_example_{language}.xlsx")
     doc.export_pdf(f"unogenerator_example_{language}.pdf")
     doc.close()
 
+
+
+
     return f"unogenerator_example_{language}.ods took {datetime.now()-doc.init}"
     
     
 def demo_odt_standard(language):
     if language=="en":
-        lang1=gettext.install('unogenerator', 'badlocale')
+        lang1=install('unogenerator', 'badlocale')
     else:
-        lang1=gettext.translation('unogenerator', 'unogenerator/locale', languages=[language])
+        lang1=translation('unogenerator', 'unogenerator/locale', languages=[language])
         lang1.install()
 
     doc=ODT_Standard()
@@ -220,20 +241,22 @@ doc=ODT()"""    , "Code")
     
 
     doc.addParagraph(_("Tables"), "Heading 2")
-    doc.addParagraph(_("We can create tables too, for example with size 11pt:"), "Standard")
     table_data=[
         [_("Concept"), _("Value") ], 
         [_("Text"), _("This is a text")], 
-#        [_("Datetime"), datetime.now()], 
-#        [_("Date"), date.today()], 
-#        [_("Decimal"), Decimal("12.121")], 
-#        [_("Currency"), Currency(12.12, "EUR")], 
-#        [_("Percentage"), Percentage(1, 3)], 
+        [_("Datetime"), datetime.now()], 
+        [_("Date"), date.today()], 
+        [_("Float"),  12.121], 
+        [_("Currency"), Currency(-12.12, "EUR")], 
+        [_("Percentage"), Percentage(1, 3)], 
     ]
     
     
-    doc.addTableParagraph(table_data)
-    doc.pageBreak()
+    doc.addParagraph(_("We can create tables with diferent font sizes and formats:"), "Standard")
+    doc.addTableParagraph(table_data, columnssize_percentages=[30, ])
+    
+    doc.addTableParagraph(table_data, columnssize_percentages=[30, ],  size=6, style="3D")
+    
 
     doc.addParagraph(_("Lists and numbered lists"), "Heading 2") 
     doc.addParagraph(_("Simple list"), "Standard")
@@ -263,6 +286,11 @@ doc=ODT()"""    , "Code")
     doc.addParagraph(_("The next paragraph is generated with the illustration method"), "Standard")
     doc.addImageParagraph([pkg_resources.resource_filename(__name__, 'images/crown.png')]*5, 2500, 1500, "Illustration")
 
+
+    doc.addParagraph(_("Search and Replace"), "Heading 2")
+    doc.addParagraph(_("Below this paragraph is a paragraph with a % REPLACEME % (Without white spaces) text and it's going to be replaced after all document is been generated"), "Standard")
+    doc.addParagraph("%REPLACEME%", "Standard")
+
     doc.pageBreak()
     doc.addParagraph(_("ODS"), "Heading 1")
     doc.addParagraph("""    TTO READ
@@ -274,6 +302,11 @@ def demo_ods_standard_read():
     doc.close()
     return "demo_ods_standard took {}".format(datetime.now()-doc.init)""",  "Standard")
     
+    doc.find_and_replace_and_setcursorposition("%REPLACEME%")
+    doc.addParagraph(_("This paragraph was set at the end of the code after a find and replace command."), "Standard")
+    
+    
+    
     doc.save(f"unogenerator_documentation_{language}.odt")
     doc.export_docx(f"unogenerator_documentation_{language}.docx")
     doc.export_pdf(f"unogenerator_documentation_{language}.pdf")
