@@ -8,13 +8,24 @@ from uno import getComponentContext, createUnoStruct
 from com.sun.star.beans import PropertyValue
 from com.sun.star.text import ControlCharacter
 from com.sun.star.awt import Size
+from com.sun.star.style.ParagraphAdjust import RIGHT,  LEFT
 from com.sun.star.style.BreakType import PAGE_AFTER
+from gettext import translation
+from logging import warning
 from pkg_resources import resource_filename
-from unogenerator.commons import Coord as C, Colors,  Range as R, datetime2uno, row2index, column2index
+from unogenerator.commons import Coord as C, ColorsNamed,  Range as R, datetime2uno, guess_object_style, row2index, column2index
 from unogenerator.reusing.currency import Currency
 from unogenerator.reusing.datetime_functions import string2dtnaive, string2date
 from unogenerator.reusing.percentage import Percentage
 
+
+try:
+    t=translation('unogenerator', resource_filename("unogenerator","locale"))
+    _=t.gettext
+except:
+    _=str
+
+
 class ODF:
     def __init__(self, template=None, loserver_port=2002):
         self.template=None if template is None else f"file://{path.abspath(template)}"
@@ -34,7 +45,7 @@ class ODF:
     def print_styles(self):
         stylefam=self.document.StyleFamilies
         
-        print(f"Document '{self.filename}' styles")
+        print("Document styles")
         for sf_index,  sf in enumerate(stylefam):
             print (f"  * Family '{stylefam.getElementNames()[sf_index]}'")
             styles=list(sf.getElementNames())
@@ -67,8 +78,17 @@ class ODT(ODF):
         self.cursor=self.document.Text.createTextCursor()
 
         
-    def save(self, filename):
+    def save(self, filename, overwrite_template=False):
         filename=f"file://{path.abspath(filename)}"
+        if filename==self.template and overwrite_template is False:
+            print(_("You can't use the same filename as your template or you will overwrite it."))
+            print(_("You can force it, setting overwrite_template paramter to True"))
+            print(_("Document hasn't been saved."))
+            return
+        if filename.endswith(".odt") is False:
+            print(_("Filename extension must be 'odt'."))
+            print(_("Document hasn't been saved."))
+            return
         ## SAVE FILE
         args=(
             PropertyValue('FilterName',0,'writer8',0),
@@ -79,20 +99,37 @@ class ODT(ODF):
                 
     def export_pdf(self, filename):
         filename=f"file://{path.abspath(filename)}"
+        if filename.endswith(".pdf") is False:
+            print(_("Filename extension must be 'pdf'."))
+            print(_("Document hasn't been saved."))
+            return
         args=(
             PropertyValue('FilterName',0,'writer_pdf_Export',0),
             PropertyValue('Overwrite',0,True,0),
         )
         self.document.storeToURL(filename, args)
-        
-    def find_and_remove_and_setcursorposition(self, find):
+         
+##def insertTextIntoCell( table, cellName, text, color ):
+##    tableText = table.getCellByName( cellName )
+##    cursor = tableText.createTextCursor()
+##    cursor.setPropertyValue( "CharColor", color )
+##    tableText.setString( text )
+    def find_and_replace_and_setcursorposition(self, find, replace=""):
         search=self.document.createSearchDescriptor()
         search.SearchString=find
         found=self.document.findFirst(search)
-        found.String.replace(find,  "")
+        if found is not None:
+            found.setString(replace)
+            self.cursor=found
+        else:
+            warning(f"'{find}' was not found in the document'")
 
     def export_docx(self, filename):
         filename=f"file://{path.abspath(filename)}"
+        if filename.endswith(".docx") is False:
+            print(_("Filename extension must be 'docx'."))
+            print(_("Document hasn't been saved."))
+            return
         args=(
             PropertyValue('FilterName',0,'MS Word 2007 XML',0),
             PropertyValue('Overwrite',0,True,0),
@@ -131,37 +168,72 @@ class ODT(ODF):
         for filename in filename_list:
             self.document.Text.insertTextContent(self.cursor,self.textcontentImage(filename, width, height, "AS_CHARACTER"), False)
         self.document.Text.insertControlCharacter(self.cursor, ControlCharacter.PARAGRAPH_BREAK, False)
-        
-##    table.setPropertyValue( "BackTransparent", uno.Bool(0) )
-##    table.setPropertyValue( "BackColor", 13421823 )
-##    row = rows.getByIndex(0)
-##    row.setPropertyValue( "BackTransparent", uno.Bool(0) )
-##    row.setPropertyValue( "BackColor", 6710932 )
-##    table.getCellByName("C4").setValue(415.7)
-##    table.getCellByName("D4").setFormula("sum <A4:C4>")
- 
-##def insertTextIntoCell( table, cellName, text, color ):
-##    tableText = table.getCellByName( cellName )
-##    cursor = tableText.createTextCursor()
-##    cursor.setPropertyValue( "CharColor", color )
-##    tableText.setString( text )
-    def addTableParagraph(self, data, paragraph_style="Standard", table_style="Elegant"):
-        ## Table by cellname
+
+
+    ## Table data, must be all strings (Not a spreadsheet)
+    ## @params magings TRBL
+    ## @params columnssize_percentage Por ejemplo para table 30,70, solo debo poner [30,]
+    ##   para table [10,10,80], solo debo poner [10,10]
+    ## if none , rangos iguales
+    def addTableParagraph(self, 
+        data,  
+        columnssize_percentages=None, 
+        margins_top_bottom=500,  
+        size=10,  
+        width_percentage=80, 
+        alignment="center",  
+        paragraph_style="Standard", 
+        style=None, 
+        name=None
+    ):        
+        #Tabla definition
+        table=self.document.createInstance("com.sun.star.text.TextTable")
         num_rows=len(data)
         if num_rows==0:
             return
         num_columns=len(data[0])
-        self.cursor.setPropertyValue("ParaStyleName", paragraph_style)
-        table=self.document.createInstance("com.sun.star.text.TextTable")
-#        table.setPropertyValue("TableStyleName", table_style)
         table.initialize(num_rows, num_columns)
+        if name is not None:
+            table.TableName=name
+
+        #Párrafo de la table
+        self.cursor.setPropertyValue("ParaStyleName", paragraph_style)
         self.document.Text.insertTextContent(self.cursor,table,False)
+        self.document.Text.insertControlCharacter(self.cursor, ControlCharacter.PARAGRAPH_BREAK, False)
+        
+        #Table data, must be all strings (Not a spreadsheet)
+        if style is not None:
+            table.TableTemplateName=style #MUST BE BEFORE TO OVERRIDE SOME PROPERTIES
         for row, row_data in enumerate(data):
             for column, cell_data in enumerate(row_data):
-                cell=table.getCellByPosition(row, column)
-                cell.setString(str(cell_data))
-        #cell.insertTextContent(cell,image,False)
-        self.document.Text.insertControlCharacter(self.cursor, ControlCharacter.PARAGRAPH_BREAK, False)
+                cell=table.getCellByPosition( column, row)
+                cursor = cell.createTextCursor()
+                cursor.CharHeight=size
+                if str(cell_data).startswith("-"):
+                    cursor.setPropertyValue( "CharColor", 0xDD0000 )
+                if cell_data.__class__.__name__ in ["str"]:
+                    cursor.ParaAdjust=LEFT
+                else:
+                    cursor.ParaAdjust=RIGHT
+                    cursor.ParaRightMargin=100
+                cell.insertString(cursor, str(cell_data), False)
+        
+        #TAble width and style
+        table.HoriOrient=2 #Centered
+        table.TopMargin=margins_top_bottom
+        table.BottomMargin=margins_top_bottom
+        table.RelativeWidth=width_percentage #PARECE QUE ES SOLO DESCRIPTIVO
+        
+        #Columns width
+        if columnssize_percentages is not None:
+            separators=[]
+            for sep in columnssize_percentages:
+                separator= createUnoStruct("com.sun.star.text.TableColumnSeparator")
+                separator.Position=sep*100
+                separator.IsVisible=True
+                separators.append(separator)
+            #        print(table.TableColumnRelativeSum)
+            table.TableColumnSeparators=separators
         
     def addListPlain(self, arr, list_style="List_2", paragraph_style="Puntitos"):
 #        def get_items(list_o, list_style, paragraph_style):
@@ -222,14 +294,13 @@ class ODS(ODF):
             
     def setComment(self, coord, comment):
         coord=C.assertCoord(coord)
-#        print(cell.supportsService("com.sun.star.sheet.XSheetAnnotation"))
         celladdress= createUnoStruct("com.sun.star.table.CellAddress")
         celladdress.Sheet=self.sheet_index
         celladdress.Column=coord.letterIndex()
         celladdress.Row=coord.numberIndex()
         self.sheet.Annotations.insertNew(celladdress, comment)
             
-    def addCell(self, coord, o, color_dict=Colors["White"], outlined=1, alignment="left", decimals=2, bold=False):
+    def addCell(self, coord, o, color_dict=ColorsNamed.White, outlined=1, alignment="left", decimals=2, bold=False):
         coord=C.assertCoord(coord)
         cell=self.sheet.getCellByPosition(coord.letterIndex(), coord.numberIndex())
         self.__set_value_to_cell(cell, o)
@@ -242,12 +313,57 @@ class ODS(ODF):
         cell.setPropertyValue("BottomBorder", border_prop)
         cell.setPropertyValue("CellBackColor", color_dict["color"])
 
-    def addCellWithStyle(self, coord, o, color_dict=Colors["White"], style=None):
+    ## @param colors If None uses Wh
+    ## @param styles If None uses guest style. Else an array of styles
+    def addRowWithStyle(self, coord_start, list_o, colors=ColorsNamed.White,styles=None):
+        coord_start=C.assertCoord(coord_start)
+        if styles is None:
+            styles=[]
+            for o in list_o:
+                styles.append(guess_object_style(o))
+
+        if colors.__class__.__name__!="list":
+            colors=[colors]*len(list_o)
+
+        for i,o in enumerate(list_o):
+            self.addCellWithStyle(coord_start.addColumnCopy(i),o,colors[i],styles[i])
+
+    ## @param colors If None uses Wh
+    ## @param styles If None uses guest style. Else an array of styles
+    def addColumnWithStyle(self, coord_start, list_o, colors=ColorsNamed.White,styles=None):
+        coord_start=C.assertCoord(coord_start)
+        if styles is None:
+            styles=[]
+            for o in list_o:
+                styles.append(guess_object_style(o))
+
+        if colors.__class__.__name__!="list":
+            colors=[colors]*len(list_o)
+
+        for i,o in enumerate(list_o):
+            self.addCellWithStyle(coord_start.addRowCopy(i),o,colors[i],styles[i])
+
+    ## @param style If None tries to guess it
+    def addListOfRowsWithStyle(self, coord_start, list_rows, colors=ColorsNamed.White, styles=None):
+        coord_start=C.assertCoord(coord_start)
+        for i, row in enumerate(list_rows):
+            self.addRowWithStyle(coord_start.addRowCopy(i), row, colors=colors,styles=styles)
+
+    ## @param style If None tries to guess it
+    def addListOfColumnsWithStyle(self, coord_start, list_columns, colors=ColorsNamed.White, styles=None):
+        coord_start=C.assertCoord(coord_start)
+        for i, column in enumerate(list_columns):
+            self.addColumnWithStyle(coord_start.addColumnCopy(i), column, colors=colors,styles=styles)
+
+    ## @param style If None tries to guess it
+    def addCellWithStyle(self, coord, o, color=ColorsNamed.White, style=None):
         coord=C.assertCoord(coord)
+        if style is None:
+            style=guess_object_style(o)
         cell=self.sheet.getCellByPosition(coord.letterIndex(), coord.numberIndex())
         self.__set_value_to_cell(cell, o)
         cell.setPropertyValue("CellStyle", style)
-        cell.setPropertyValue("CellBackColor", color_dict["color"])
+        cell.setPropertyValue("CellBackColor", color)
         
     def __set_value_to_cell(self, cell, o):
         if o.__class__.__name__  == "datetime":
@@ -273,14 +389,14 @@ class ODS(ODF):
             cell.setString(str(o))
             print("MISSING", o.__class__.__name__)
         
-    def addCellMerged(self, range, o, color_dict=Colors["White"], style=None):
+    def addCellMergedWithStyle(self, range, o, color=ColorsNamed.White, style=None):
         range=R.assertRange(range)
         cell=self.sheet.getCellByPosition(range.start.letterIndex(), range.start.numberIndex())
         cellrange=self.sheet.getCellRangeByName(range.string())
         cellrange.merge(True)
         self.__set_value_to_cell(cell, o)
         cell.setPropertyValue("CellStyle", style)
-        cell.setPropertyValue("CellBackColor", color_dict["color"])
+        cell.setPropertyValue("CellBackColor", color)
 
     def freezeAndSelect(self, freeze, current=None, topleft=None):
         freeze=C.assertCoord(freeze) 
@@ -374,9 +490,8 @@ class ODS(ODF):
     ## Return a Range object with the limits of the index sheet
     def getSheetRange(self):
         endcoord=C("A1").addRow(self.rowNumber()-1).addColumn(self.columnNumber()-1)
-        return R("A1:"+endcoord.string())
-        
-        
+        return R("A1:" + endcoord.string())
+
     ## Esta función puede que sea costosa
     def rowNumber(self):
         return len(self.sheet.getData())
@@ -388,17 +503,30 @@ class ODS(ODF):
         else:
             return len(self.sheet.getData()[0])
             
-    def save(self, filename):
+    def save(self, filename, overwrite_template=False):
         filename=f"file://{path.abspath(filename)}"
+        if filename==self.template and overwrite_template is False:
+            print(_("You can't use the same filename as your template or you will overwrite it."))
+            print(_("You can force it, setting overwrite_template paramter to True"))
+            print(_("Document hasn't been saved."))
+            return
+        if filename.endswith(".ods") is False:
+            print(_("Filename extension must be 'ods'."))
+            print(_("Document hasn't been saved."))
+            return
         ## SAVE FILE
         args=(
-            PropertyValue('FilterName',0,'calc8',0),
-            PropertyValue('Overwrite',0,True,0),
+            PropertyValue('FilterName', 0, 'calc8', 0),
+            PropertyValue('Overwrite', 0, True, 0),
         )
         self.document.storeAsURL(filename, args)
 
     def export_pdf(self, filename):
         filename=f"file://{path.abspath(filename)}"
+        if filename.endswith(".pdf") is False:
+            print(_("Filename extension must be 'pdf'."))
+            print(_("Document hasn't been saved."))
+            return
         args=(
             PropertyValue('FilterName',0,'calc_pdf_Export',0),
             PropertyValue('Overwrite',0,True,0),
@@ -407,13 +535,16 @@ class ODS(ODF):
 
     def export_xlsx(self, filename):
         filename=f"file://{path.abspath(filename)}"
+        if filename.endswith(".xlsx") is False:
+            print(_("Filename extension must be 'xlsx'."))
+            print(_("Document hasn't been saved."))
+            return
         args=(
             PropertyValue('FilterName',0,'Calc MS Excel 2007 XML',0),
             PropertyValue('Overwrite',0,True,0),
         )
         self.document.storeToURL(filename, args)
 
-
 class ODS_Standard(ODS):
     def __init__(self, loserver_port=2002):
         ODS.__init__(self, resource_filename(__name__, 'templates/standard.ods'), loserver_port)
